/* @TITLE "barrier.h: interface for barrier.ca" */
/*
 * barrier - a simple tree-based message-passing barrier 
 * ANSI C
 *
 * This is a simple tree-based barrier. 
 *
 * Part of 
 *           The STARFISH Parallel file-system simulator
 *      (Simulation Tool for Advanced Research in File Systems)
 *
 *                              David Kotz
 *                          Dartmouth College
 *                             Version 2.0
 *                             January 1995
 *                         dfk@cs.dartmouth.edu
 *
 * $Id: barrier.h,v 2.0 95/01/18 12:18:17 dfk RELEASE $
 */

#ifndef BARRIER_H
#define BARRIER_H

#include "dfk.h"	      /* get the basic types */

/* Called to initialize the data structures.
 * The range of processors involved is provided.
 * This means that processors [low..high] are involved.
 * This may be called at any time when the barrier is inactive,
 * to reset low and high. After the first initialization, it 
 * doesn't hurt to have many identical calls to barrier_init.
 */
extern void barrier_init(int low_proc, int high_proc);

/* BARRIER: Called by all participating procs at a barrier. */
extern void barrier(void);

/* SPLIT BARRIER:
 * a split barrier has the root of the tree return after synchrony,
 * but before releasing others. split_barrier
 * returns TRUE if you are the one (the root). 
 * After taking care of any business, you must then call barrier_release.
 */
extern boolean split_barrier(void);
extern void barrier_release(void);

#endif /* BARRIER_H */

